/*
 * GXMame
 *
 * 2002, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <stdio.h>
#include <stdlib.h>
#include "callbacks.h"
#include "interface.h"
#include "about.h"
#include "directories.h"
#include "mameio.h"
#include "support.h"
#include "gxmame.h"
#include "audit.h"
#include "properties.h"
#include "io.h"

static GtkWidget *AboutWindow;
guint timeoutid;

gboolean delayed_select_game(gpointer game_data);
void select_game(RomEntry *rom);

/*Close the main window*/
gboolean
on_MainWindow_delete_event             (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	exit_gxmame();
	return TRUE;
}


void
on_exit_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	exit_gxmame();
}


void
on_about_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (AboutWindow)
	{
		gdk_window_raise(AboutWindow->window);
	}
	else
	{
		AboutWindow = create_AboutWindow ();
		gtk_widget_show(AboutWindow);
		set_icon(AboutWindow, _("About"));	
	}
  
}


/*About window*/
gboolean
on_AboutWindow_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_widget_hide(AboutWindow);
  gtk_widget_destroy(AboutWindow);
  AboutWindow = NULL;
  return FALSE;
}

void
on_aboutButtonOK_clicked_event         (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_widget_destroy(AboutWindow);
	AboutWindow = NULL;
}


/* Toolbox menu*/
void
on_filterShowButton_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
	{
		gxmame_debug("Show filters");
		/* the on_folder_list_activate callback will call show_filter()*/
		gtk_check_menu_item_set_active(main_gui.folder_list_menu,TRUE);
	}
	else
	{
		gxmame_debug("Hide filters");
		/* the on_folder_list_activate callback will call hide_filter()*/
		gtk_check_menu_item_set_active(main_gui.folder_list_menu,FALSE);
	}
}


void
on_snapShowButton_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
	{
		gxmame_debug("Show snaps");
		gtk_check_menu_item_set_active(main_gui.screen_shot_menu,TRUE);
	}
	else
	{
		gxmame_debug("Hide snaps");
		gtk_check_menu_item_set_active(main_gui.screen_shot_menu,FALSE);
	}
}


void
on_mode_button_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
	{
		if (gui_prefs.current_mode!=GPOINTER_TO_INT(user_data))
		{
			gui_prefs.previous_mode=gui_prefs.current_mode;
			gui_prefs.current_mode=GPOINTER_TO_INT(user_data);
			gxmame_debug("current mode changed to:%d,previous changed to :%d",gui_prefs.current_mode,gui_prefs.previous_mode);
			switch(gui_prefs.current_mode)
			{
				case(LARGE_ICONS):
					gtk_check_menu_item_set_active(main_gui.large_icons_view_menu,TRUE);
					break;
				case(SMALL_ICONS):
					gtk_check_menu_item_set_active(main_gui.small_icons_view_menu,TRUE);
					break;
				case(LIST):
					gtk_check_menu_item_set_active(main_gui.list_view_menu,TRUE);
					break;
				case(DETAILS):
					gtk_check_menu_item_set_active(main_gui.details_view_menu,TRUE);
					break;
			}
			switch(gui_prefs.previous_mode)
			{
				case(LARGE_ICONS):
					gtk_toggle_button_set_active(main_gui.large_icons_view_button,FALSE);
					break;
				case(SMALL_ICONS):
					gtk_toggle_button_set_active(main_gui.small_icons_view_button,FALSE);
					break;
				case(LIST):
					gtk_toggle_button_set_active(main_gui.list_view_button,FALSE);
					break;
				case(DETAILS):
					gtk_toggle_button_set_active(main_gui.details_view_button,FALSE);
					break;
			}
			create_gamelist_content();
		}		
	}
	else
	{
		if (gui_prefs.current_mode==GPOINTER_TO_INT(user_data))
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),TRUE);
	}
}


void
on_ModifyThe_toggled                   (GtkToggleButton *button,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
	{
		gui_prefs.ModifyThe=1;
		if (gui_prefs.DefaultGame_p ->the_trailer)
			g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s, The %s",gui_prefs.DefaultGame_p->gamename,gui_prefs.DefaultGame_p->gamenameext);
		else
			g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s %s",gui_prefs.DefaultGame_p->gamename,gui_prefs.DefaultGame_p->gamenameext);
		g_strstrip(gui_prefs.DefaultGame);
		/* Need to sort the glist if sorted by name*/
		roms = g_list_sort(roms,(GCompareFunc )compare_game_name);
		create_gamelist_content();
	}
	else
	{
		gui_prefs.ModifyThe=0;
		if (gui_prefs.DefaultGame_p->the_trailer)
			g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"The %s %s",gui_prefs.DefaultGame_p->gamename,gui_prefs.DefaultGame_p->gamenameext);
		else
			g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s %s",gui_prefs.DefaultGame_p->gamename,gui_prefs.DefaultGame_p->gamenameext);
		g_strstrip(gui_prefs.DefaultGame);
		/* Need to sort the glist if sorted by name*/
		roms = g_list_sort(roms,(GCompareFunc )compare_game_name);
		create_gamelist_content();
	}
}


void
on_folder_selected                     (GtkCTree        *CTree,
                                        GtkCTreeNode    *CTreeNode,
                                        gint            *column,
                                        gpointer         user_data)
{
	simple_filter *current_folder_filter;
	current_folder_filter = (simple_filter *)gtk_ctree_node_get_row_data(CTree,CTreeNode);
	if (!(current_folder_filter->FolderID == MANUFACTURERS ||
	      current_folder_filter->FolderID == YEARS ||
	      current_folder_filter->FolderID == CUSTOMS ||
	      current_folder_filter->FolderID == CATEGORIES ||
	      current_folder_filter->FolderID == VERSIONS))
	{
		gtk_ctree_expand(CTree,CTreeNode);
		current_filter = current_folder_filter;
		gui_prefs.previous_mode = gui_prefs.current_mode;
		create_gamelist_content();
	}
	gxmame_debug("filter:%i selected",current_folder_filter->FolderID);
	gui_prefs.FolderID = current_folder_filter->FolderID;
}


void
on_folder_unselected                   (GtkCTree        *CTree,
                                        GtkCTreeNode    *CTreeNode,
                                        gint            *column,
                                        gpointer         user_data)
{
	simple_filter *current_folder_filter;
	current_folder_filter = (simple_filter *)gtk_ctree_node_get_row_data(CTree,CTreeNode);
	if (!(current_folder_filter->FolderID == MANUFACTURERS ||
	      current_folder_filter->FolderID == YEARS ||
	      current_folder_filter->FolderID == CUSTOMS ||
	      current_folder_filter->FolderID == CATEGORIES ||
	      current_folder_filter->FolderID == VERSIONS))
	{
		gtk_ctree_collapse(CTree,CTreeNode);
	}
}

void
on_row_selected                       (GtkCList        *CList,
                                        gint             game,
                                        gint             column,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	gchar *message[1];
	RomEntry *game_data;
	game_data = gtk_clist_get_row_data(CList,game);
	gtk_clist_get_pixtext (CList,game,0, message,NULL,NULL,NULL);
	if (event && event->type == GDK_2BUTTON_PRESS) {
		if (event->button == 1) {
			gxmame_debug("double clic on: %s",message[0]);
			launch_game(game_data);
		}
	} else if (event && event->type == GDK_BUTTON_PRESS) {
		if (event->button == 3) {
			gxmame_debug("game %s selected (row# %d)(right button)",message[0],game);
			select_game(game_data);
			gtk_menu_popup(main_gui.popup_menu,NULL,NULL,
					NULL,NULL,event->button,event->time);
		} else {
			gxmame_debug("game %s selected (row# %d)",message[0],game);
			select_game(game_data);
		}
	}else {
		/* do we need that? mst see the global speed */
		/* Call rom_selected after a timeout */
		if (timeoutid)
			gtk_timeout_remove(timeoutid);
		timeoutid =
		    gtk_timeout_add(SELECT_TIMEOUT,
				    (GtkFunction) delayed_select_game, game_data);
	}
}


gboolean delayed_select_game(gpointer game_data)
{
	select_game(game_data);
	gtk_timeout_remove(timeoutid);
	return FALSE;
}

void
on_play_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	launch_game(gui_prefs.DefaultGame_p);
}


void
on_select_random_game_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{	
	GtkWidget *displayed_list;
	gint random_game;
	displayed_list = GTK_BIN(main_gui.scrolled_window_games)->child;
	/* the max should be the number of element in the current gtkCList*/
	random_game = (gint)(visible_games * ((gfloat)rand()/ RAND_MAX));
	gxmame_debug("random game#%i",random_game);
	gtk_clist_select_row(GTK_CLIST(displayed_list),random_game,0);
	gtk_clist_moveto(GTK_CLIST(displayed_list),random_game,0,0.5,0);

}


void
on_add_to_favorites_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gui_prefs.DefaultGame_p->favourite = TRUE;
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.add_to_favorites),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.remove_from_favorites),TRUE);
	/* problems because the row values are completly changed, I redisplay the complete game list
	update_game_in_list((gui_prefs.DefaultGame_p)->position);*/
	if (current_filter->type == FAVORITE)
		create_gamelist_content();
}


void
on_remove_from_favorites_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gui_prefs.DefaultGame_p->favourite = FALSE;
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.add_to_favorites),TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.remove_from_favorites),FALSE);
	/* problems because the row values are completly changed, I redisplay the complete game list
	update_game_in_list((gui_prefs.DefaultGame_p)->position);*/
	if (current_filter->type == FAVORITE)
		create_gamelist_content();
}


void
on_folder_list_activate                (GtkCheckMenuItem *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show filters menu");
		/* shouldn't I block the signal to toggle button 
		   otherwise, infinite loop? 
		   it seems it's working anyway ??
		   maybe the gtk_toggle_button_set_active doesn't provoke a call
		   to on_mode_button_clicked*/
		gtk_toggle_button_set_active (main_gui.filterShowButton,TRUE);
		show_filters();
	}
	else
	{
		gxmame_debug("Hide filters menu");
		gtk_toggle_button_set_active (main_gui.filterShowButton,FALSE);
		hide_filters();
	}

}


void
on_screen_shot_activate                (GtkCheckMenuItem *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show snaps menu");
		gtk_toggle_button_set_active (main_gui.snapShowButton,TRUE);
		show_snaps();
	}
	else
	{
		gxmame_debug("Hide snaps menu");
		gtk_toggle_button_set_active (main_gui.snapShowButton,FALSE);
		hide_snaps();
	}
}


void
on_refresh_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static gboolean quick_check_running;
	GList *list_pointer;
	RomEntry *rom;
	
if(!quick_check_running)
{	/*prevent user to launch several quick check at the same time */
	quick_check_running=1;
	/* remove all information concerning the presence of roms */
	for (list_pointer = g_list_first(roms);(list_pointer != NULL);)
	{
		rom = (RomEntry *)list_pointer->data;
		rom->has_roms = 2;
		list_pointer = g_list_next(list_pointer);
	}
	/* refresh the display */
	create_gamelist_content();
	not_checked_list = g_list_copy (roms);
	
	quick_check();
	/* should put a refresh single rom here */
	/* final refresh*/
	create_gamelist_content();
	quick_check_running=0;
}
else gxmame_debug("Quick check already running");
}


void
on_large_icons_menu_activate           (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show large icons menu");
		gtk_toggle_button_set_active (main_gui.large_icons_view_button,TRUE);
	}
	else
	{
		gxmame_debug("Hide large icons menu");
		gtk_toggle_button_set_active (main_gui.large_icons_view_button,FALSE);
	}
}


void
on_small_icons_view_menu_activate      (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show small icons menu");
		gtk_toggle_button_set_active (main_gui.small_icons_view_button,TRUE);
	}
	else
	{
		gxmame_debug("Hide small icons menu");
		gtk_toggle_button_set_active (main_gui.small_icons_view_button,FALSE);
	}

}


void
on_list_view_menu_activate             (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show list menu");
		gtk_toggle_button_set_active (main_gui.list_view_button,TRUE);
	}
	else
	{
		gxmame_debug("Hide list menu");
		gtk_toggle_button_set_active (main_gui.list_view_button,FALSE);
	}

}


void
on_details_view_menu_activate          (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show details menu");
		gtk_toggle_button_set_active (main_gui.details_view_button,TRUE);
	}
	else
	{
		gxmame_debug("Hide details menu");
		gtk_toggle_button_set_active (main_gui.details_view_button,FALSE);
	}

}


void
on_toolbar_view_menu_activate          (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show toolbar");
		show_toolbar();
	}
	else
	{
		gxmame_debug("Hide toolbar");
		hide_toolbar();
	}
}


void
on_status_bar_view_menu_activate       (GtkCheckMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (menuitem->active)
	{
		gxmame_debug("Show status bar");
		show_status_bar();
	}
	else
	{
		gxmame_debug("Hide status bar");
		hide_status_bar();
	}
}



void
update_ctree_filter_width              (GtkCTree        *ctree,
                                        GList           *node,
                                        gpointer         user_data)
{
	gtk_clist_columns_autosize(GTK_CLIST(ctree));
	gtk_widget_queue_draw (GTK_WIDGET(ctree));
}


void
on_rebuild_game_list_menu_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gxmame_debug("freeing glists");
	g_list_foreach(roms, (GFunc)g_free, NULL);
//	g_list_foreach(roms, free_game_glist_element, NULL);
	g_list_free (roms);
	roms = NULL;
	/* free manufacturers and years list */
	g_list_foreach(manufacturers, (GFunc)g_free, NULL);
	g_list_free (manufacturers);
	manufacturers = NULL;
	g_list_free (years);
	years = NULL;
	g_list_foreach(categories, (GFunc)g_free, NULL);
	g_list_free (categories);
	categories = NULL;
	g_list_foreach(versions, (GFunc)g_free, NULL);
	g_list_free (versions);
	versions = NULL;
	
	/* I thought it could free my memory but nope*/
	while (gtk_events_pending()) gtk_main_iteration();
	g_blow_chunks();
	while (gtk_events_pending()) gtk_main_iteration();
	
	/* just to show an empty list */
	create_gamelist_content();
	
	/* reload them */
	/* free category and version list */
	/* reload them */
	gxmame_debug("recreate game list");
	create_game_list_file();
	gxmame_debug("reload everything");
	load_games_list();
	load_games_ini();
	load_catver_ini();
	create_filtersTreeContent();
	create_gamelist_content();
}


void
on_directories_menu_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *directory_window;
	directory_window = create_directories_selection();
	gtk_widget_show(directory_window);
	set_icon(directory_window,_("Directories selection"));
}

void
on_audit_all_games_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	create_checking_games_window();
	while (gtk_events_pending()) gtk_main_iteration();
	launch_checking_games_window_1();
	
}


void
on_properties_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *properties_window;
	/* have to test if a game is selected or not
	   then only after lauche the properties window*/
	properties_window = create_properties_windows();
	gtk_widget_show(properties_window);
	set_icon(properties_window,_("Properties"));
}

