/*
 * GXMame
 *
 * 2002, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "gxmame.h"
#include "gui.h"
#include "icones.h"
#include "progression_window.h"
#include <stdio.h>
#include <libintl.h>

#include "io.h"

#include <gdk-pixbuf/gdk-pixbuf.h>

void create_Toolbar (void);
void select_game(RomEntry *rom);

void init_gui()
{
	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;
	GdkColor transparent;
	char *window_title;
	gchar *message;
	GtkWidget *Loading_error = NULL;
	
	GtkAccelGroup *popup_menu_accels;
  guint tmp_key;
//  GtkWidget *play;
  GtkWidget *separator1;
//  GtkWidget *add_to_favorites;
  GtkWidget *separator2;
//  GtkWidget *remove_from_favorites;
  GtkWidget *select_random_game;
  GtkWidget *separator3;
  GtkWidget *properties;
  GtkAccelGroup *accel_group;
  GtkTooltips *tooltips;

  tooltips = gtk_tooltips_new ();

  accel_group = gtk_accel_group_new ();

	MainWindow = create_MainWindow ();
	/* realize the main window to be able to create pixmaps with colormaps */
	gtk_widget_realize(MainWindow);
	
	if (list_version.version==NULL)
	{
		Loading_error = GXMame_error_message(_("problem to recognise the gamelist version \nyou'd better build a new one\ngo to Option and rebuild a game list"));
	} else if (!strcmp(list_version.version,"unknown"))
	{
		Loading_error = GXMame_error_message(_("Could not recognize format of gamelist.\nYou need to build a new gamelist."));
	} else if (!strcmp(list_version.version,"too old"))
	{
		Loading_error = GXMame_error_message(_("Gamelist from too old GXMame\nThe format of the gamelist has changed in this version.\nYou need to build a new gamelist."));
	} else if (strcmp(executable_version.name,list_version.name)||strcmp(executable_version.version,list_version.version))
	{
		message = g_strdup_printf(_("the gamelist is from:\n%s (%s) %s\nand the current executable is:\n%s (%s) %s\nyou may want rebuild the game list"),
				executable_version.name,
				executable_version.target,
				executable_version.version,
				list_version.name,
				list_version.target,
				list_version.version);
		Loading_error = GXMame_error_message(message);
	}
	
	/* menu */
	main_gui.play_menu = GTK_MENU_ITEM(lookup_widget(MainWindow, "play"));
	main_gui.add_to_favorites = GTK_MENU_ITEM(lookup_widget(MainWindow, "add_to_favorites"));
	main_gui.remove_from_favorites = GTK_MENU_ITEM(lookup_widget(MainWindow, "remove_from_favorites"));
	main_gui.folder_list_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "folder_list"));
	main_gui.screen_shot_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "screen_shot"));
	main_gui.toolbar_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "toolbar_view_menu"));
	main_gui.status_bar_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "status_bar_view_menu"));
	
	main_gui.large_icons_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "large_icons_view_menu"));
	main_gui.small_icons_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "small_icons_view_menu"));
	main_gui.list_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "list_view_menu"));
	main_gui.details_view_menu = GTK_CHECK_MENU_ITEM(lookup_widget(MainWindow, "details_view_menu"));
	
	/* left pane (filters) */
	main_gui.hpanedLeft = GTK_PANED(lookup_widget(MainWindow, "hpanedLeft"));
	/* central scrolled window (games list) */
	main_gui.scrolled_window_filters = GTK_WIDGET(lookup_widget(MainWindow, "scrolledwindowFilters"));
	main_gui.scrolled_window_games = GTK_WIDGET(lookup_widget(MainWindow, "scrolledwindowGames"));
	main_gui.hpanedRight = GTK_PANED(lookup_widget(MainWindow, "hpanedRight"));
	main_gui.hseparator2 = GTK_WIDGET(lookup_widget(MainWindow, "hseparator2"));
	main_gui.toolbar = GTK_TOOLBAR(lookup_widget(MainWindow, "toolbar1"));
	main_gui.hseparator1 = GTK_WIDGET(lookup_widget(MainWindow, "hseparator1"));
	main_gui.ctree_filters = GTK_CTREE(lookup_widget(MainWindow, "ctreeFilters"));
	main_gui.screenshot_hist_frame = GTK_FRAME(lookup_widget(MainWindow, "ScreenShot_hist_frame"));
	main_gui.screenshot_hist_vbox = GTK_BOX(lookup_widget(MainWindow, "ScreenShot_hist_vbox"));
	main_gui.tri_status_bar = GTK_WIDGET(lookup_widget(MainWindow, "tri_status_bar"));
	main_gui.combo_progress_bar = GTK_WIDGET(lookup_widget(MainWindow, "combo_progress_bar"));
	main_gui.status_progress_bar = GTK_STATUSBAR(lookup_widget(MainWindow, "status_progress_bar"));
	main_gui.progress_progress_bar = GTK_PROGRESS_BAR(lookup_widget(MainWindow, "progress_progress_bar"));
	main_gui.statusbar1 = GTK_STATUSBAR(lookup_widget(MainWindow, "statusbar1"));
	main_gui.statusbar2 = GTK_STATUSBAR(lookup_widget(MainWindow, "statusbar2"));
	main_gui.statusbar3 = GTK_STATUSBAR(lookup_widget(MainWindow, "statusbar3"));
	
	/* need to create the menu to have all button in the toolbar */
	create_Toolbar ();
	
	main_gui.filterShowButton = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "filterShowButton"));
	main_gui.snapShowButton = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "snapShowButton"));
	main_gui.large_icons_view_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "large_icons_view_button"));
	main_gui.small_icons_view_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "small_icons_view_button"));
	main_gui.list_view_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "list_view_button"));
	main_gui.details_view_button = GTK_TOGGLE_BUTTON(lookup_widget(MainWindow, "details_view_button"));	

	create_filtersTreeContent();
	gtk_paned_set_position (main_gui.hpanedLeft, gui_prefs.Splitters[0]);
	gtk_paned_set_position (main_gui.hpanedRight, gui_prefs.Splitters[1]);

#ifndef DEBUG	
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.large_icons_view_menu),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.list_view_menu),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.large_icons_view_button),FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.list_view_button),FALSE);
#endif
	
	gtk_widget_hide (GTK_WIDGET(main_gui.combo_progress_bar));
	
	gtk_check_menu_item_set_active(main_gui.toolbar_view_menu,gui_prefs.ShowToolBar);
	gtk_check_menu_item_set_active(main_gui.status_bar_view_menu,gui_prefs.ShowStatusBar);
	main_gui.scrolled_window_filters->allocation.width = gui_prefs.Splitters[0];
	gtk_check_menu_item_set_active(main_gui.folder_list_menu,gui_prefs.ShowFolderList);
	/* need this line or GTK will allocate a value for the gamelist that will mess the screenshot display*/
	main_gui.scrolled_window_games->allocation.width = gui_prefs.Splitters[1];
	gtk_check_menu_item_set_active(main_gui.screen_shot_menu,gui_prefs.ShowScreenShot);
	
	gtk_window_set_default_size(GTK_WINDOW(MainWindow),
				    gui_prefs.GUIWidth,
				    gui_prefs.GUIHeight);
	gtk_widget_set_uposition(MainWindow,
				    gui_prefs.GUIPosX,
				    gui_prefs.GUIPosY);
	
	/* display the default picture */
	testpix = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&testmask,
					&transparent,
					mame_xpm);
	main_gui.testgtkpix = gtk_pixmap_new (testpix,testmask);

	gtk_box_pack_start(main_gui.screenshot_hist_vbox,main_gui.testgtkpix,TRUE,TRUE,5);
	gtk_widget_show (main_gui.testgtkpix);
#ifdef DEBUG	
	main_gui.history_box = gtk_text_new(NULL,NULL);
	gtk_box_pack_start(main_gui.screenshot_hist_vbox,main_gui.history_box,TRUE,TRUE,5);
	gtk_widget_show (main_gui.history_box);
#endif


	/* build the popup menu */
	main_gui.popup_menu = GTK_MENU(gtk_menu_new ());
	gtk_widget_ref (GTK_WIDGET(main_gui.popup_menu));
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "popup_menu", main_gui.popup_menu,
				  (GtkDestroyNotify) gtk_widget_unref);
//	gtk_menu_item_set_submenu (GTK_MENU_ITEM (file_menu), main_gui.popup_menu);
	popup_menu_accels = gtk_menu_ensure_uline_accel_group (GTK_MENU (main_gui.popup_menu));

	main_gui.popup_play_menu = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.popup_play_menu )->child),
					 _("_Play"));
	gtk_widget_add_accelerator (main_gui.popup_play_menu, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (main_gui.popup_play_menu);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "play", main_gui.popup_play_menu,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_gui.popup_play_menu);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), main_gui.popup_play_menu);
	gtk_tooltips_set_tip (tooltips, main_gui.popup_play_menu, _("Play currently selected game"), NULL);
	gtk_widget_add_accelerator (main_gui.popup_play_menu, "activate", accel_group,
				    GDK_Return, 0,
				    GTK_ACCEL_LOCKED);
  	gtk_signal_connect (GTK_OBJECT (main_gui.popup_play_menu), "activate",
                      	    GTK_SIGNAL_FUNC (on_play_activate),
                            NULL);

	separator1 = gtk_menu_item_new ();
	gtk_widget_ref (separator1);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "separator1", separator1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (separator1);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), separator1);
	gtk_widget_set_sensitive (separator1, FALSE);

	main_gui.popup_add_to_favorites = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.popup_add_to_favorites)->child),
					 _("A_dd to 'Favorites'"));
	gtk_widget_add_accelerator (main_gui.popup_add_to_favorites, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (main_gui.popup_add_to_favorites);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "add_to_favorites", main_gui.popup_add_to_favorites,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_gui.popup_add_to_favorites);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), main_gui.popup_add_to_favorites);
	gtk_tooltips_set_tip (tooltips, main_gui.popup_add_to_favorites, _("Add this game to your 'Favorites' game folder"), NULL);
	gtk_signal_connect (GTK_OBJECT (main_gui.popup_add_to_favorites), "activate",
			    GTK_SIGNAL_FUNC (on_add_to_favorites_activate),
			    NULL);

	main_gui.popup_remove_from_favorites = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.popup_remove_from_favorites)->child),
					 _("R_emove from 'Favorites'"));
	gtk_widget_add_accelerator (main_gui.popup_remove_from_favorites, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (main_gui.popup_remove_from_favorites);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "remove_from_favorites", main_gui.popup_remove_from_favorites,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (main_gui.popup_remove_from_favorites);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), main_gui.popup_remove_from_favorites);
	gtk_tooltips_set_tip (tooltips, main_gui.popup_remove_from_favorites, _("Remove this game from your 'Favorites' game folder"), NULL);
	gtk_signal_connect (GTK_OBJECT (main_gui.popup_remove_from_favorites), "activate",
			    GTK_SIGNAL_FUNC (on_remove_from_favorites_activate),
			    NULL);

	separator2 = gtk_menu_item_new ();
	gtk_widget_ref (separator2);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "separator2", separator2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (separator2);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), separator2);
	gtk_widget_set_sensitive (separator2, FALSE);

	select_random_game = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (select_random_game)->child),
					 _("_Select Random Game"));
	gtk_widget_add_accelerator (select_random_game, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (select_random_game);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "select_random_game", select_random_game,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (select_random_game);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), select_random_game);
	gtk_tooltips_set_tip (tooltips, select_random_game, _("Randomly select a game"), NULL);
	gtk_signal_connect (GTK_OBJECT (select_random_game), "activate",
			    GTK_SIGNAL_FUNC (on_select_random_game_activate),
			    NULL);

	separator3 = gtk_menu_item_new ();
	gtk_widget_ref (separator3);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "separator3", separator3,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (separator3);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), separator3);
	gtk_widget_set_sensitive (separator3, FALSE);

	properties = gtk_menu_item_new_with_label ("");
	tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (properties)->child),
					 _("Pr_operties"));
	gtk_widget_add_accelerator (properties, "activate_item", popup_menu_accels,
				    tmp_key, 0, 0);
	gtk_widget_ref (properties);
	gtk_object_set_data_full (GTK_OBJECT (MainWindow), "properties", properties,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (properties);
	gtk_container_add (GTK_CONTAINER (main_gui.popup_menu), properties);
	gtk_tooltips_set_tip (tooltips, properties, _("Display the properties of the selected items"), NULL);
	gtk_signal_connect (GTK_OBJECT (properties), "activate",
			    GTK_SIGNAL_FUNC (on_properties_activate),
			    NULL);
	
	
	/* Main window title and icon */	
	window_title = g_strdup_printf(_("GXMame V%s running %s (%s) %s"),
				VERSION,
				executable_version.name,
				executable_version.target,
				executable_version.version);
	gtk_window_set_title (GTK_WINDOW (MainWindow), window_title);
	g_free(window_title);
	set_icon(MainWindow,"GXMame");
	
	/* need to show the window before creating the CList
	   or the first moveto wont apear correctly */
	gtk_widget_show (MainWindow);
	if (Loading_error != NULL )
		gdk_window_raise(Loading_error->window);
	
	switch (gui_prefs.current_mode)
	{
		case (LARGE_ICONS):
			gxmame_debug("large icons mode not implemented yet");
			break;
		case (SMALL_ICONS):
			gxmame_debug("Display small icons mode");
			display_small_icon_list();
			break;
		case (LIST):
			gxmame_debug("list mode not implemented yet");
			break;
		case (DETAILS):
			gxmame_debug("Display details mode");
			display_details_list();
			break;
	}
	/*need to set the size here otherwise it move when we create the gamelist*/
	if (gui_prefs.ShowScreenShot)
		gtk_paned_set_position (main_gui.hpanedRight, gui_prefs.Splitters[1]);

}


void create_Toolbar (void)
{
	GtkWidget *tmp_toolbar_icon;
	GtkWidget *filterShowButton;
	GtkWidget *snapShowButton;
	GtkWidget *large_icons_view_button;
	GtkWidget *small_icons_view_button;
	GtkWidget *list_view_button;
	GtkWidget *details_view_button;
	GtkWidget *modify_the_button;
	GtkWidget *aboutButton;
	
	GdkPixmap *tmp_pixmap_icon;
	GdkBitmap *tmp_mask_icon;
	GdkColor  transparent;
	

/* addition of toolbar buttons */
/* filters and snapshots buttons */
	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					filtermenu_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        filterShowButton = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Show Filters"),
        			      _("Toggle Folder List"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (filterShowButton);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "filterShowButton", filterShowButton,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (filterShowButton);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (filterShowButton), gui_prefs.ShowFolderList);

	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					snapmenu_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        snapShowButton = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Show Snaps"),
        			      _("Toggle Screen Shot"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (snapShowButton);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "snapShowButton", snapShowButton,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (snapShowButton);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (snapShowButton), gui_prefs.ShowScreenShot);

        gtk_toolbar_append_space (main_gui.toolbar);

/* listing mode buttons */
	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					viewlarge_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        large_icons_view_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Large Icons"),
        			      _("Large Icons"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (large_icons_view_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "large_icons_view_button", large_icons_view_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (large_icons_view_button);

	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					viewsmall_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        small_icons_view_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Small Icons"),
        			      _("Small Icons"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (small_icons_view_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "small_icons_view_button", small_icons_view_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (small_icons_view_button);

	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					viewlist_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        list_view_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("List"),
        			      _("List"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (list_view_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "list_view_button", list_view_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (list_view_button);

	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					viewdetail_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        details_view_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Details"),
        			      _("Details"), NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (details_view_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "details_view_button", details_view_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (details_view_button);

/* init the mode button */	
	switch(gui_prefs.current_mode)
	{
		case(LARGE_ICONS):
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(large_icons_view_button), TRUE);
			break;
 		case(SMALL_ICONS):
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(small_icons_view_button), TRUE);
			break;
 		case(LIST):
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(list_view_button), TRUE);
			break;
 		case(DETAILS):
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(details_view_button), TRUE);
			break;
	}

        gtk_toolbar_append_space (main_gui.toolbar);

/* the trailer button */
	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					thetrail_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
        modify_the_button = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_TOGGLEBUTTON,
        			      NULL,
        			      _("Modify The"),
        			      _("Modify The"), NULL,
        			      tmp_toolbar_icon, NULL , NULL);
        gtk_widget_ref (modify_the_button);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "modify_the_button", modify_the_button,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (modify_the_button);
	
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(modify_the_button), gui_prefs.ModifyThe);


        gtk_toolbar_append_space (main_gui.toolbar);

/* About button */
	tmp_pixmap_icon = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&tmp_mask_icon,
					&transparent,
					about_xpm);
	tmp_toolbar_icon = gtk_pixmap_new (tmp_pixmap_icon,tmp_mask_icon);
	gdk_pixmap_unref(tmp_pixmap_icon);
	gdk_pixmap_unref(tmp_mask_icon);
        aboutButton = gtk_toolbar_append_element (main_gui.toolbar,
        			      GTK_TOOLBAR_CHILD_BUTTON,
        			      NULL,
        			      _("About"),
        			      NULL, NULL,
        			      tmp_toolbar_icon, NULL, NULL);
        gtk_widget_ref (aboutButton);
        gtk_object_set_data_full (GTK_OBJECT (MainWindow), "aboutButton", aboutButton,
        			  (GtkDestroyNotify) gtk_widget_unref);
        gtk_widget_show (aboutButton);
 
/* Connection of toolbar buttons signals*/
        gtk_signal_connect (GTK_OBJECT (filterShowButton), "toggled",
        		    GTK_SIGNAL_FUNC (on_filterShowButton_toggled),
        		    NULL);
        gtk_signal_connect (GTK_OBJECT (snapShowButton), "toggled",
        		    GTK_SIGNAL_FUNC (on_snapShowButton_toggled),
        		    NULL);
        gtk_signal_connect (GTK_OBJECT (large_icons_view_button), "clicked",
        		    GTK_SIGNAL_FUNC (on_mode_button_clicked),
        		    GINT_TO_POINTER(LARGE_ICONS));
        gtk_signal_connect (GTK_OBJECT (small_icons_view_button), "clicked",
        		    GTK_SIGNAL_FUNC (on_mode_button_clicked),
        		    GINT_TO_POINTER(SMALL_ICONS));
        gtk_signal_connect (GTK_OBJECT (list_view_button), "clicked",
        		    GTK_SIGNAL_FUNC (on_mode_button_clicked),
        		    GINT_TO_POINTER(LIST));
        gtk_signal_connect (GTK_OBJECT (details_view_button), "clicked",
        		    GTK_SIGNAL_FUNC (on_mode_button_clicked),
        		    GINT_TO_POINTER(DETAILS));
        gtk_signal_connect (GTK_OBJECT (modify_the_button), "toggled",
        		    GTK_SIGNAL_FUNC(on_ModifyThe_toggled),
        		    GINT_TO_POINTER(DETAILS));
        gtk_signal_connect (GTK_OBJECT (aboutButton), "clicked",
        		    GTK_SIGNAL_FUNC (on_about_activate),
        		    NULL);
}


void create_filtersTreeContent (void)
{
	GtkCTreeNode *node;
	GtkCTreeNode *node2;
	GtkCTreeNode *current_node_selected;
	gchar *text[1];
	GList *listpointer;
	gint folder_ID_selected = NUMBER_FOLDER;
	static gboolean firstrun = TRUE;
	
	GdkPixmap *pixmap_folder_open;
	GdkBitmap *mask_folder_open;
	GdkPixmap *pixmap_folder_close;
	GdkBitmap *mask_folder_close;
	GdkColor  transparent;
	
	simple_filter *folder_filter;
	
	
	gtk_clist_freeze(GTK_CLIST(main_gui.ctree_filters));
	
	/* disconnect signal of previous CTree to prevent a crash when I clean this Ctree
	   should not be done during the first run
	  I'm just adding a static variable, I'm sure I can do by an other way but I''m too tired to search*/
	if (!firstrun)
	{
		gtk_signal_disconnect_by_func(GTK_OBJECT (main_gui.ctree_filters),
					GTK_SIGNAL_FUNC (on_folder_selected),
					NULL);
		gtk_signal_disconnect_by_func(GTK_OBJECT (main_gui.ctree_filters),
					GTK_SIGNAL_FUNC (on_folder_unselected),
					NULL);
	}
	firstrun=FALSE;

	/* clean the CTree*/
	gtk_ctree_post_recursive(main_gui.ctree_filters,
				NULL,
				destroy_simple_filter,
				NULL);

	/* we will need to sort year, category, version and manufacturer*/
	gtk_clist_set_compare_func(GTK_CLIST(main_gui.ctree_filters),NULL);
	
	
	gtk_ctree_set_line_style(main_gui.ctree_filters,
                           GTK_CTREE_LINES_DOTTED);
	gtk_clist_set_row_height(GTK_CLIST(main_gui.ctree_filters), 16);
	gtk_clist_set_selection_mode(GTK_CLIST(main_gui.ctree_filters), GTK_SELECTION_BROWSE);
 	pixmap_folder_open = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_open,
 				&transparent,
 				foldopen_xpm);
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				folder_xpm);
	/* Filter's Tree*/
	/* ALL GAMES */
 	text[0] = g_strdup_printf(_("All Games"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = ALL;
//	if (gui_prefs.FolderID==folder_filter->FolderID)
	/* default folder selected */
	current_filter=	folder_filter;
	current_node_selected = node;
	/* but expand it only if it was the last selected*/
	if (gui_prefs.FolderID==folder_filter->FolderID)
		gtk_ctree_expand(main_gui.ctree_filters,node);
	
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* AVAILABLES */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				foldavail_xpm);
 	text[0] = g_strdup_printf(_("Available"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = HAS_ROMS;
	folder_filter->is = TRUE;
	folder_filter->value = GINT_TO_POINTER(1);
	folder_filter->FolderID = AVAILABLE;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* UNAVAILABLE */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				foldunav_xpm);
 	text[0] = g_strdup_printf(_("Unavailable"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = HAS_ROMS;
	folder_filter->is = FALSE;
	folder_filter->value = GINT_TO_POINTER(1);
	folder_filter->FolderID = UNAVAILABLE;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* NEOGEO */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				neogeo_xpm);
 	text[0] = g_strdup_printf(_("Neo-Geo"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("neogeo");
	folder_filter->FolderID = NEOGEO;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CPS1 */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				cps_xpm);
 	text[0] = g_strdup_printf(_("CPS1"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("cps1");
	folder_filter->FolderID = CPS1;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CPS2 */
 	text[0] = g_strdup_printf(_("CPS2"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("cps2");
	folder_filter->FolderID = CPS2;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* MANUFACTURERS */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				foldmanu_xpm);
 	text[0] = g_strdup_printf(_("Manufacturer"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = MANUFACTURERS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	
	/* MANUFACTURERS LIST */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				manufact_xpm);
/* insert all manufacturers */
	for(listpointer = g_list_first(manufacturers);
			(listpointer != NULL);
			listpointer = g_list_next(listpointer))
	{
		simple_filter *manufilter;
		text[0] = g_strdup_printf("%s",(gchar *)listpointer->data);
 		node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 					      text, 5,
 					      pixmap_folder_close,
 					      mask_folder_close,
 					      pixmap_folder_open,
 				     	      mask_folder_open,
 				 	      FALSE, FALSE);
		manufilter = g_new(simple_filter,1);
		manufilter->type = MANU;
		manufilter->is = TRUE;
		manufilter->value = listpointer->data;
//		manufilter->value = (gpointer *)g_strdup_printf("%s",(gchar *)listpointer->data);
		manufilter->FolderID = folder_ID_selected++;
		if (gui_prefs.FolderID==manufilter->FolderID)
		{
			current_filter = manufilter;
			current_node_selected = node2;
			gtk_ctree_expand(main_gui.ctree_filters,node);
			gtk_ctree_expand(main_gui.ctree_filters,node2);
		}
		gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,manufilter);
	}
	gtk_ctree_sort_node(main_gui.ctree_filters,node);

	/* YEARS */
	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				foldyear_xpm);
 	text[0] = g_strdup_printf(_("Year"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = YEARS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	
	/* YEARS LIST */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				year_xpm);
/* insert all years */
	for(listpointer = g_list_last(years);
				(listpointer != NULL);
				listpointer = g_list_previous(listpointer))
	{
		simple_filter *yearfilter;
		if (listpointer->data == 0)
		{
			text[0] = g_strdup_printf(_("<unknown>"));
		}
		else
		{
			text[0] = g_strdup_printf("%d",GPOINTER_TO_INT(listpointer->data));
		}
 		node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 					      text, 5,
 					      pixmap_folder_close,
 					      mask_folder_close,
 					      pixmap_folder_open,
 					      mask_folder_open,
 					      FALSE, FALSE);
		yearfilter = g_new(simple_filter,1);
		yearfilter->type = YEAR;
		yearfilter->is = TRUE;
		yearfilter->value = listpointer->data;
		yearfilter->FolderID = folder_ID_selected++;
		if (gui_prefs.FolderID==yearfilter->FolderID)
		{
			current_filter = yearfilter;
			current_node_selected = node2;
			gtk_ctree_expand(main_gui.ctree_filters,node);
			gtk_ctree_expand(main_gui.ctree_filters,node2);
		}
		gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,yearfilter);
	}
	gtk_ctree_sort_node(main_gui.ctree_filters,node);

	/* WORKING */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				working_xpm);
 	text[0] = g_strdup_printf(_("Working"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = STATUS;
	folder_filter->is = TRUE;
	folder_filter->value = CORRECT;
	folder_filter->FolderID = WORKING;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* NONWORKING */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				nonwork_xpm);
 	text[0] = g_strdup_printf(_("Non-Working"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = STATUS;
	folder_filter->is = FALSE;
	folder_filter->value = CORRECT;
	folder_filter->FolderID = NONWORKING;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CUSTOM */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				custom_xpm);
 	text[0] = g_strdup_printf(_("Custom"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = CUSTOMS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	
	/* FAVORITES */
 	text[0] = g_strdup_printf(_("Favorites"));
 	node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = FAVORITE;
	folder_filter->is = TRUE;
	folder_filter->value = NULL;
	folder_filter->FolderID = FAVORITES;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node2;
		gtk_ctree_expand(main_gui.ctree_filters,node);
		gtk_ctree_expand(main_gui.ctree_filters,node2);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,folder_filter);

	/*PLAYED  */
 	text[0] = g_strdup_printf(_("Played"));
 	node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = TIMESPLAYED;
	folder_filter->is = FALSE;
	folder_filter->value = 0;
	folder_filter->FolderID = PLAYED;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node2;
		gtk_ctree_expand(main_gui.ctree_filters,node);
		gtk_ctree_expand(main_gui.ctree_filters,node2);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,folder_filter);

	/* ORIGINALS */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				folder_xpm);
 	text[0] = g_strdup_printf(_("Originals"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = CLONE;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("-");
	folder_filter->FolderID = ORIGINALS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CLONES */
 	text[0] = g_strdup_printf(_("Clones"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = CLONE;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf("-");
	folder_filter->FolderID = CLONES;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* RASTERS */
 	text[0] = g_strdup_printf(_("Raster"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = VECTOR;
	folder_filter->is = FALSE;
	folder_filter->value = NULL;
	folder_filter->FolderID = RASTERS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	text[0] = g_strdup_printf(_("Vector"));
	/* VECTORS */
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = VECTOR;
	folder_filter->is = TRUE;
	folder_filter->value = NULL;
	folder_filter->FolderID = VECTORS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* TRACKBALL */
 	text[0] = g_strdup_printf(_("Trackball"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = CONTROL;
	folder_filter->is = TRUE;
	folder_filter->value = (gpointer *)g_strdup_printf("trackball");
	folder_filter->FolderID = TRACKBALL;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* STEREO */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				sound_xpm);
 	text[0] = g_strdup_printf(_("Stereo"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = CHANNELS;
	folder_filter->is = TRUE;
	folder_filter->value = GINT_TO_POINTER(2);
	folder_filter->FolderID = STEREO;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
		gtk_ctree_expand(main_gui.ctree_filters,node);
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);

	/* CATEGORY */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				foldcat_xpm);
 	text[0] = g_strdup_printf(_("Category"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = CATEGORIES;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	/* insert all categories */
	for(listpointer = g_list_first(categories);
				(listpointer != NULL);
				listpointer = g_list_next(listpointer))
	{
		simple_filter *catfilter;
		text[0] = g_strdup(listpointer->data);
 		node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 					      text, 5,
 					      pixmap_folder_close,
 					      mask_folder_close,
 					      pixmap_folder_open,
 					      mask_folder_open,
 					      FALSE, FALSE);
		catfilter = g_new(simple_filter,1);
		catfilter->type = CATEGORY;
		catfilter->is = TRUE;
		catfilter->value = listpointer->data;
		catfilter->FolderID = folder_ID_selected++;
		if (gui_prefs.FolderID==catfilter->FolderID)
		{
			current_filter = catfilter;
			current_node_selected = node2;
			gtk_ctree_expand(main_gui.ctree_filters,node);
			gtk_ctree_expand(main_gui.ctree_filters,node2);
		}
		gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,catfilter);
	}
	gtk_ctree_sort_node(main_gui.ctree_filters,node);

	/* VERSION */
 	pixmap_folder_close = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_folder_close,
 				&transparent,
 				foldvers_xpm);
 	text[0] = g_strdup_printf(_("Version"));
 	node = gtk_ctree_insert_node(main_gui.ctree_filters, NULL, NULL,
 				      text, 5,
 				      pixmap_folder_close,
 				      mask_folder_close,
 				      pixmap_folder_open,
 				      mask_folder_open,
 				      FALSE, FALSE);
	folder_filter = g_new(simple_filter,1);
	folder_filter->type = DRIVER;
	folder_filter->is = FALSE;
	folder_filter->value = (gpointer *)g_strdup_printf(" ");
	folder_filter->FolderID = VERSIONS;
	if (gui_prefs.FolderID==folder_filter->FolderID)
	{
		current_filter = folder_filter;
		current_node_selected = node;
	}		
	gtk_ctree_node_set_row_data(main_gui.ctree_filters,node,folder_filter);
	/* insert all version */
	for(listpointer = g_list_first(versions);
				(listpointer != NULL);
				listpointer = g_list_next(listpointer))
	{
		simple_filter *verfilter;
		text[0] = g_strdup(listpointer->data);
 		node2 = gtk_ctree_insert_node(main_gui.ctree_filters, node, NULL,
 					      text, 5,
 					      pixmap_folder_close,
 					      mask_folder_close,
 					      pixmap_folder_open,
 					      mask_folder_open,
 					      FALSE, FALSE);
		verfilter = g_new(simple_filter,1);
		verfilter->type = MAMEVER;
		verfilter->is = TRUE;
		verfilter->value = listpointer->data;
		verfilter->FolderID = folder_ID_selected++;
		if (gui_prefs.FolderID==verfilter->FolderID)
		{
			current_filter = verfilter;
			current_node_selected = node2;
			gtk_ctree_expand(main_gui.ctree_filters,node);
			gtk_ctree_expand(main_gui.ctree_filters,node2);
		}
		gtk_ctree_node_set_row_data(main_gui.ctree_filters,node2,verfilter);
	}
	gtk_ctree_sort_node(main_gui.ctree_filters,node);

	
	
 	g_free(text[0]); 
	gdk_pixmap_unref(pixmap_folder_close);
	gdk_pixmap_unref(mask_folder_close);
	gdk_pixmap_unref(pixmap_folder_open);
	gdk_pixmap_unref(mask_folder_open);
	
	/* needed here or the size is not completly right */
	gtk_clist_thaw(GTK_CLIST(main_gui.ctree_filters));
	gtk_clist_columns_autosize(GTK_CLIST(main_gui.ctree_filters));
//	while (gtk_events_pending()) gtk_main_iteration();

	/* need to connect the CTree only after because the gamelist is not yet created at this point */
	gtk_ctree_select (main_gui.ctree_filters,current_node_selected);
	/* where are you going ??*/
	gtk_ctree_node_moveto (main_gui.ctree_filters,current_node_selected,0,0,0);
	
/* Signals Connection*/
	gtk_signal_connect (GTK_OBJECT (main_gui.ctree_filters), "tree_select_row",
        		    GTK_SIGNAL_FUNC (on_folder_selected),
        		    NULL);
        gtk_signal_connect (GTK_OBJECT (main_gui.ctree_filters), "tree_unselect_row",
        		    GTK_SIGNAL_FUNC (on_folder_unselected),
        		    NULL);

}

void hide_filters (void)
{
	gui_prefs.ShowFolderList=0;
	gui_prefs.Splitters[0] = main_gui.scrolled_window_filters->allocation.width;
	gtk_paned_set_position (main_gui.hpanedLeft, 0);
	gtk_paned_set_gutter_size (main_gui.hpanedLeft, 0);
	gtk_container_remove(GTK_CONTAINER(main_gui.hpanedLeft),GTK_WIDGET(main_gui.scrolled_window_filters));
}

void show_filters (void)
{
	gui_prefs.ShowFolderList=1;
	gtk_paned_set_position (main_gui.hpanedLeft, gui_prefs.Splitters[0]);
	gtk_paned_set_gutter_size (main_gui.hpanedLeft, 6);
	gtk_paned_pack1 (main_gui.hpanedLeft, GTK_WIDGET(main_gui.scrolled_window_filters), FALSE, FALSE);
}

void hide_snaps (void)
{	
	gui_prefs.ShowScreenShot=0;
	gui_prefs.Splitters[1] = main_gui.scrolled_window_games->allocation.width;
	gtk_paned_set_position (main_gui.hpanedRight, -1);
	gtk_paned_set_gutter_size (main_gui.hpanedRight, 0);
	gtk_paned_set_handle_size(main_gui.hpanedRight, 0);
	gtk_container_remove(GTK_CONTAINER(main_gui.hpanedRight),GTK_WIDGET(main_gui.screenshot_hist_frame));
}

void show_snaps (void)
{
	gui_prefs.ShowScreenShot=1;
	gtk_paned_set_position (main_gui.hpanedRight, gui_prefs.Splitters[1]);
	gtk_paned_set_gutter_size (main_gui.hpanedRight, 6);
	gtk_paned_set_handle_size(main_gui.hpanedRight, 10);
	gtk_paned_pack2 (main_gui.hpanedRight, GTK_WIDGET(main_gui.screenshot_hist_frame), FALSE, FALSE);
}


void hide_toolbar (void)
{
	gui_prefs.ShowToolBar=0;
	gtk_widget_hide (GTK_WIDGET(main_gui.hseparator2));
	gtk_widget_hide (GTK_WIDGET(main_gui.toolbar));
	gtk_widget_hide (GTK_WIDGET(main_gui.hseparator1));
}


void show_toolbar (void)
{
	gui_prefs.ShowToolBar=1;
	gtk_widget_show (GTK_WIDGET(main_gui.hseparator2));
	gtk_widget_show (GTK_WIDGET(main_gui.toolbar));
	gtk_widget_show (GTK_WIDGET(main_gui.hseparator1));
}


void hide_status_bar (void)
{
	gui_prefs.ShowStatusBar=0;
	gtk_widget_hide (GTK_WIDGET(main_gui.tri_status_bar));
}


void show_status_bar (void)
{
	gui_prefs.ShowStatusBar=1;
	gtk_widget_show (GTK_WIDGET(main_gui.tri_status_bar));
}


void display_small_icon_list (void)
{
/* Unable to use a gtkList here cause of the 32k pixels limitation
   forced to use a gtkCList but the gtkList should work with GTK 2
   perhaps could remplace it just by hidding columns other than game name
   in the details view (if this one was previously selected)
*/
	RomEntry *tmprom;
	GList *listpointer;
	GtkWidget *displayed_list;
	gchar **Texte;
	gchar *message;
	int i=0,j=0;
	int currentgame_row=-1;
	GdkPixmap *pixmap_unknown;
	GdkBitmap *mask_unknown;
	GdkPixmap *pixmap_rom;
	GdkBitmap *mask_rom;
	GdkPixmap *pixmap_norom;
	GdkBitmap *mask_norom;
	GdkPixmap *pixmap_redx;
	GdkBitmap *mask_redx;
	GdkColor  transparent;
	
	Texte = g_malloc(sizeof(gchar *));

 	pixmap_unknown = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_unknown,
 				&transparent,
 				unknownS_xpm);
 	pixmap_rom = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_rom,
 				&transparent,
 				romS_xpm);
 	pixmap_norom = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_norom,
 				&transparent,
 				noromsS_xpm);
 	pixmap_redx = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_redx,
 				&transparent,
 				redxS_xpm);

	displayed_list = gtk_clist_new(1);
	gtk_clist_set_selection_mode(GTK_CLIST(displayed_list), GTK_SELECTION_BROWSE);
	gtk_container_add(GTK_CONTAINER(main_gui.scrolled_window_games), displayed_list);

	gtk_clist_freeze(GTK_CLIST(displayed_list));
		
	for (listpointer = g_list_first(roms);(listpointer != NULL);listpointer = g_list_next(listpointer))
	{
		tmprom = (RomEntry *) listpointer->data;
		if (game_filtered(tmprom)){
			/* dont really need this string , an empty one could be enough
			   in that case mallog ans snprintf*/
			if (tmprom->the_trailer && gui_prefs.ModifyThe)
				Texte[0] = g_strstrip(g_strdup_printf("%s, The %s",tmprom->gamename,tmprom->gamenameext));
			else if (tmprom->the_trailer && !gui_prefs.ModifyThe)
				Texte[0] = g_strstrip(g_strdup_printf("The %s %s",tmprom->gamename,tmprom->gamenameext));
			else
				Texte[0] = g_strstrip(g_strdup_printf("%s %s",tmprom->gamename,tmprom->gamenameext));
			tmprom->position = i = gtk_clist_append(GTK_CLIST(displayed_list), Texte);
			if (tmprom->has_roms == 2)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      i,
	                                      0,
	                                      Texte[0],
	                                      5,
	                                      pixmap_unknown,
	                                      mask_unknown);
			else if (tmprom->has_roms == 0)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      i,
	                                      0,
	                                      Texte[0],
	                                      5,
	                                      pixmap_norom,
	                                      mask_norom);
			else if (!tmprom->status)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      i,
	                                      0,
	                                      Texte[0],
	                                      5,
	                                      pixmap_redx,
	                                      mask_redx);
			else
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      i,
	                                      0,
	                                      Texte[0],
	                                      5,
	                                      pixmap_rom,
	                                      mask_rom);
			gtk_clist_set_row_data(GTK_CLIST(displayed_list),i,tmprom);
			
			if (!strncmp(Texte[0],gui_prefs.DefaultGame,MAX_GAMENAME))
				currentgame_row=i; 
			g_free(Texte[0]);
			/*allows to display the correct number of games*/
			i++;
		}
		else
			tmprom->position = -1;
	}
	gdk_pixmap_unref(pixmap_unknown);
	gdk_pixmap_unref(mask_unknown);
	gdk_pixmap_unref(pixmap_rom);
	gdk_pixmap_unref(mask_rom);
	gdk_pixmap_unref(pixmap_norom);
	gdk_pixmap_unref(mask_norom);
	gdk_pixmap_unref(pixmap_redx);
	gdk_pixmap_unref(mask_redx);

	/* 3 actif buttons */
	for (j = 0; j < 3; j++)
		gtk_clist_set_button_actions(GTK_CLIST(displayed_list), j, GTK_BUTTON_SELECTS);
	
	gtk_signal_connect (GTK_OBJECT (displayed_list), "select_row",
        		    GTK_SIGNAL_FUNC (on_row_selected),
        		    NULL);
	
	gtk_clist_columns_autosize(GTK_CLIST(displayed_list));
	gtk_clist_thaw(GTK_CLIST(displayed_list));

	gtk_widget_show_all(main_gui.scrolled_window_games);
	
	visible_games = i;
	message = g_strdup_printf("%d %s",i,i>1?_("games"):_("game"));
	gtk_statusbar_pop (main_gui.statusbar3, 1);
	gtk_statusbar_push (main_gui.statusbar3, 1, message);
	g_free(message);
	
	/* allow to correctly update the page_size and page_increment of the GtkAdjustment of the GtkClist*/
	while (gtk_events_pending()) gtk_main_iteration();
	if (currentgame_row>0)
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),currentgame_row,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),currentgame_row,0,0.5,0);
	}
	else
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),0,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),0,0,0,0);
	}
	main_gui.displayed_list = displayed_list;
}


void display_details_list (void)
{
	RomEntry *tmprom;
	GList *listpointer,*current_column;
  	gint i;
	GtkWidget *displayed_list;
	gchar **Textes = NULL;
	int currentgame_row=-1;
	/*FIXME this is just to display the game position properly
	  when I got the strange 0.0 position bug*/
	static gboolean first_run = TRUE;

	GdkPixmap *pixmap_unknown;
	GdkBitmap *mask_unknown;
	GdkPixmap *pixmap_rom;
	GdkBitmap *mask_rom;
	GdkPixmap *pixmap_norom;
	GdkBitmap *mask_norom;
	GdkPixmap *pixmap_redx;
	GdkBitmap *mask_redx;
	GdkColor  transparent;

	gchar *message;
	int j=0;

	Textes = g_new(gchar *,g_list_length(Columns));
	
 	pixmap_unknown = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_unknown,
 				&transparent,
 				unknownS_xpm);
 	pixmap_rom = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_rom,
 				&transparent,
 				romS_xpm);
 	pixmap_norom = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_norom,
 				&transparent,
 				noromsS_xpm);
 	pixmap_redx = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_redx,
 				&transparent,
 				redxS_xpm);
	
	for(current_column = g_list_first(Columns),i=0;
			(current_column != NULL);
			current_column = g_list_next(current_column),i++)
	{
		switch (GPOINTER_TO_INT(current_column->data))
		{
			case (GAMENAME):
				Textes[i] = g_strdup(_("Game"));
				break;
			case (HAS_ROMS):
				Textes[i] = g_strdup(_("ROMs"));
				break;
			case (HAS_SAMPLES):
				Textes[i] = g_strdup(_("Samples"));
				break;
			case (ROMNAME):
				Textes[i] = g_strdup(_("Directory"));
				break;
			case (VECTOR):
				Textes[i] = g_strdup(_("Type"));
				break;
			case (CONTROL):
				Textes[i] = g_strdup(_("Trackball"));
				break;
			case (TIMESPLAYED):
				Textes[i] = g_strdup(_("Played"));
				break;
			case (MANU):
				Textes[i] = g_strdup(_("Manufacturer"));
				break;
			case (YEAR):
				Textes[i] = g_strdup(_("Year"));
				break;
			case (CLONE):
				Textes[i] = g_strdup(_("Clone of"));
				break;
			case (DRIVER):
				Textes[i] = g_strdup(_("Driver"));
				break;
			case (ROMOF):
				Textes[i] = g_strdup(_("Rom of"));
				break;
			case (CATEGORY):
				Textes[i] = g_strdup(_("Category"));
				break;
			case (MAMEVER):
				Textes[i] = g_strdup(_("Version"));
				break;
		}
	}
	
	displayed_list = gtk_clist_new_with_titles(g_list_length(Columns), Textes);
	for (i=0 ; i<g_list_length(Columns) ; i++)
		g_free(Textes[i]);
	gtk_clist_set_selection_mode(GTK_CLIST(displayed_list), GTK_SELECTION_BROWSE);
	gtk_container_add(GTK_CONTAINER(main_gui.scrolled_window_games), displayed_list);

	gtk_clist_freeze(GTK_CLIST(displayed_list));

	for(listpointer = g_list_first(roms);
			(listpointer != NULL);
			 listpointer= g_list_next(listpointer))
	{
		tmprom = (RomEntry *) listpointer->data;
		if (game_filtered(tmprom)){
			for(current_column = g_list_first(Columns),i=0;
					(current_column != NULL);
					current_column = g_list_next(current_column),i++)
			{
				switch (GPOINTER_TO_INT(current_column->data))
				{
					case (GAMENAME):
						if (tmprom->the_trailer && gui_prefs.ModifyThe)
							Textes[i] = g_strstrip(g_strdup_printf("%s, The %s",tmprom->gamename,tmprom->gamenameext));
						else if (tmprom->the_trailer && !gui_prefs.ModifyThe)
							Textes[i] = g_strstrip(g_strdup_printf("The %s %s",tmprom->gamename,tmprom->gamenameext));
						else
							Textes[i] = g_strstrip(g_strdup_printf("%s %s",tmprom->gamename,tmprom->gamenameext));
						break;
					case (HAS_ROMS):
						Textes[i] = g_strndup(tmprom->has_roms?_("Yes"):_("No"), MAX_GAMENAME);
						break;
					case (HAS_SAMPLES):
						if (tmprom->nb_samples==0)
							Textes[i] = g_strndup(" ", MAX_GAMENAME);
						else 
						{
							Textes[i] = g_strndup(tmprom->has_samples?_("Yes"):_("No"), MAX_GAMENAME);
						}
						break;
					case (ROMNAME):
						Textes[i] = g_strndup(tmprom->romname, MAX_ROMNAME);
						break;
					case (VECTOR):
						Textes[i] = g_strndup(tmprom->vector?_("Vector"):_("Raster"), MAX_ROMNAME);
						break;
					case (CONTROL):
						if (!strcmp(tmprom->control,"trackball"))
							Textes[i] = g_strndup(_("Yes"), MAX_ROMNAME);
						else
							Textes[i] = g_strndup(_("No"), MAX_ROMNAME);
						break;
					case (TIMESPLAYED):
						Textes[i] = g_strdup_printf("%d", tmprom->timesplayed);
						break;
					case (MANU):
						Textes[i] = g_strndup(tmprom->manu, MAX_MANU);
						break;
					case (YEAR):
						Textes[i] = g_strndup(tmprom->year, MAX_YEAR);
						break;
					case (CLONE):
						Textes[i] = g_strndup(tmprom->cloneof, MAX_ROMNAME);
						break;
					case (DRIVER):
						Textes[i] = g_strndup(tmprom->driver, MAX_ROMNAME);
						break;
					case (ROMOF):
						Textes[i] = g_strndup(tmprom->romof, MAX_ROMNAME);
						break;
					case (CATEGORY):
						Textes[i] = g_strndup(tmprom->category, MAX_CATEGORY);
						break;
					case (MAMEVER):
						Textes[i] = g_strndup(tmprom->mame_ver_added, MAX_MAMEVER);
						break;
				}
			}
	
			tmprom->position = j = gtk_clist_append(GTK_CLIST(displayed_list), Textes);
	/* Should be the first column and not move (perhaps remove this first column from the columns list?)*/
			if (tmprom->has_roms == 2)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_unknown,
	                                      mask_unknown);
			else if (tmprom->has_roms == 0)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_norom,
	                                      mask_norom);
			else if (!tmprom->status)
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_redx,
	                                      mask_redx);
			else
				gtk_clist_set_pixtext(GTK_CLIST(displayed_list),
	                                      j,
	                                      0,
	                                      Textes[0],
	                                      5,
	                                      pixmap_rom,
	                                      mask_rom);
	
			gtk_clist_set_row_data(GTK_CLIST(displayed_list),j,tmprom);
			
			if (!strncmp(Textes[0],gui_prefs.DefaultGame,MAX_GAMENAME))
				currentgame_row=j; 
			
			for (i=0 ; i<g_list_length(Columns) ; i++)
				g_free(Textes[i]);
			/*allows to display the correct number of games*/
			j++;
		}	
		else
			tmprom->position = -1;
	}
	
	g_free(Textes);
	gdk_pixmap_unref(pixmap_unknown);
	gdk_pixmap_unref(mask_unknown);
	gdk_pixmap_unref(pixmap_rom);
	gdk_pixmap_unref(mask_rom);
	gdk_pixmap_unref(pixmap_norom);
	gdk_pixmap_unref(mask_norom);
	gdk_pixmap_unref(pixmap_redx);
	gdk_pixmap_unref(mask_redx);
	
	for(current_column = g_list_first(Columns),i=0;
			(current_column != NULL);
			current_column = g_list_next(current_column),i++)
	{
		switch (GPOINTER_TO_INT(current_column->data))
		{
			case (GAMENAME):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,200);
				break;
			case (HAS_ROMS):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (HAS_SAMPLES):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (ROMNAME):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,80);
				break;
			case (VECTOR):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (CONTROL):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,60);
				break;
			case (TIMESPLAYED):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (MANU):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,110);
				break;
			case (YEAR):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,50);
				break;
			case (CLONE):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,80);
				break;
			case (DRIVER):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,60);
				break;
			case (ROMOF):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,145);
				break;
			case (CATEGORY):
				gtk_clist_set_column_width(GTK_CLIST(displayed_list),i,100);
				break;
		}
	}
	
	/* 3 actif buttons */
	for (i = 0; i < 3; i++)
		gtk_clist_set_button_actions(GTK_CLIST(displayed_list), i, GTK_BUTTON_SELECTS);

	gtk_signal_connect (GTK_OBJECT (displayed_list), "select_row",
        		    GTK_SIGNAL_FUNC (on_row_selected),
        		    NULL);

	gtk_clist_thaw(GTK_CLIST(displayed_list));

	gtk_widget_show_all(main_gui.scrolled_window_games);

	visible_games = j;
	message = g_strdup_printf("%d %s",j,j>1?_("games"):_("game"));
	gtk_statusbar_pop (main_gui.statusbar3, 1);
	gtk_statusbar_push (main_gui.statusbar3, 1, message);
	g_free(message);

	/* allow to correctly update the page_size and page_increment of the GtkAdjustment of the GtkClist*/
	/*FIXME this is just to display the game position properly
	  when I got the strange 0.0 position bug*/
	if(!first_run || !(gui_prefs.GUIPosX && gui_prefs.GUIPosY))
		while (gtk_events_pending())
			gtk_main_iteration();
	if (currentgame_row>0)
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),currentgame_row,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),currentgame_row,0,0.5,0);
	}
	else
	{
	        gtk_clist_select_row(GTK_CLIST(displayed_list),0,0);
		gtk_clist_moveto(GTK_CLIST(displayed_list),0,0,0,0);
	}
	first_run = FALSE;
	main_gui.displayed_list = displayed_list;
}

void create_gamelist_content(void)
{
	GtkWidget *displayed_list;
	displayed_list = GTK_BIN(main_gui.scrolled_window_games)->child;
	
	/* Free the previous structures */
	/* hide, empty, remove then destroy the content of scrolled window games */
	switch (gui_prefs.previous_mode)
	{
		case (LARGE_ICONS):
			gxmame_debug("free Large icons(not implemented yet)");
			break;
		case (SMALL_ICONS):
			gxmame_debug("free Small icons");
			gtk_widget_hide(displayed_list);
			gtk_clist_clear(GTK_CLIST(displayed_list));
			gtk_container_remove(GTK_CONTAINER(main_gui.scrolled_window_games),displayed_list);
/*			not needed because gtk_container_remove the widget that only had this reference
			gtk_widget_destroy(displayed_list);*/
			break;
		case (LIST):
			gxmame_debug("free gtk_clist (not implemented yet)");
			break;
		case (DETAILS):
			gtk_widget_hide(displayed_list);
			gxmame_debug("Free gtk_clist (Details)");
			gtk_clist_clear(GTK_CLIST(displayed_list));
			gtk_container_remove(GTK_CONTAINER(main_gui.scrolled_window_games),displayed_list);
/*			not needed because gtk_container_remove the widget that only had this reference
			gtk_widget_destroy(displayed_list);*/
			break;
	}	
	
	
	/* Test if the glist need to be sorted*/
	/*if (!glist_games_sorted())
		g_list_sort (roms, compare_func);*/
	/* create the new structures */
	switch (gui_prefs.current_mode)
	{
		case (LARGE_ICONS):
			gxmame_debug("large icons mode not implemented yet");
			break;
		case (SMALL_ICONS):
			gxmame_debug("Display small icons mode");
			display_small_icon_list();
			break;
		case (LIST):
			gxmame_debug("list mode not implemented yet");
			break;
		case (DETAILS):
			gxmame_debug("Display details mode");
			display_details_list();
			break;
	}			
}

/*can remove this one */
void list_example (void)
{
	RomEntry *tmprom;
	GList *listpointer;
	GtkWidget *clist;
	gchar **Textes;

	
	GdkPixmap *pixmap_game;
	GdkBitmap *mask_game;
	GdkColor  transparent;
	
	Textes = g_malloc(sizeof(gchar *));

 	pixmap_game = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_game,
 				&transparent,
 				unknownS_xpm);
	
	/* Set up the titles for the columns */
//	Textes[0] = g_malloc(MAX_GAMENAME);
	Textes[0] = g_strndup(_("Game"),MAX_GAMENAME);
	clist = gtk_clist_new_with_titles(1, Textes);
	
	gtk_container_add(GTK_CONTAINER(main_gui.scrolled_window_games), clist);
	g_free(Textes[0]);
	Textes = g_malloc(MAX_GAMENAME);

	gtk_clist_freeze(GTK_CLIST(clist));

	Textes[0] = g_malloc0(MAX_GAMENAME);
	for(listpointer = g_list_first(roms);
			(listpointer != NULL);
			 listpointer= g_list_next(listpointer))
	{
		int j;
		tmprom = (RomEntry *) listpointer->data;
		/* the _ avoid zero-length format string warning*/
		g_snprintf(Textes[0], MAX_COLUMN_NAME,"_");
		j = gtk_clist_append(GTK_CLIST(clist), Textes);
		gtk_clist_set_pixtext(GTK_CLIST(clist),
                                      j,
                                      0,
                                      tmprom->gamename,
                                      5,
                                      pixmap_game,
                                      mask_game);
//		g_free(Textes[0]);
	}
	
	gdk_pixmap_unref(pixmap_game);
	gdk_pixmap_unref(mask_game);
	g_free(Textes[0]);
	g_free(Textes);
	
	gtk_clist_columns_autosize(GTK_CLIST(clist));
	gtk_clist_thaw(GTK_CLIST(clist));

	gtk_widget_show_all(main_gui.scrolled_window_games);

}


void set_icon(GtkWidget * widget, const gchar * name)
{
	GtkStyle *style;
	GdkBitmap *icon_mask = NULL;
	GdkPixmap *icon_pm = NULL;

	style = gtk_widget_get_style(widget);

	icon_pm = gdk_pixmap_create_from_xpm_d(widget->window,
					       &icon_mask,
					       &style->bg
					       [GTK_STATE_NORMAL],
					       (gchar **)
					       mamel_xpm);

	gdk_window_set_icon(widget->window, NULL, icon_pm, icon_mask);
	gdk_window_set_icon_name(widget->window, name);

	g_free(icon_pm);
	g_free(icon_mask);
}

void select_game(RomEntry *rom)
{
	GdkPixbuf *pixbuf;
	GdkPixmap *testpix = NULL;
	GdkPixmap *testmask = NULL;
	GdkColor transparent;
	gchar *filename;
	gchar *play_menu_name;

	
	/* Maybe I should remove the string default gamename and only keep a pointer to the rom struct ? */
	gui_prefs.DefaultGame_p = rom;
	if (rom->the_trailer && gui_prefs.ModifyThe)
		g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s, The %s",rom->gamename,rom->gamenameext);
	else if (rom->the_trailer && !gui_prefs.ModifyThe)
		g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"The %s %s",rom->gamename,rom->gamenameext);
	else
		g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s %s",rom->gamename,rom->gamenameext);

	g_strstrip(gui_prefs.DefaultGame);
	
	/* update menus */
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.add_to_favorites),!rom->favourite);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.remove_from_favorites),rom->favourite);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.popup_add_to_favorites),!rom->favourite);
	gtk_widget_set_sensitive(GTK_WIDGET(main_gui.popup_remove_from_favorites),rom->favourite);
	
	play_menu_name = g_strdup_printf(_("_Play %s"),gui_prefs.DefaultGame);
  	gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.play_menu)->child),play_menu_name);
  	gtk_label_parse_uline (GTK_LABEL (GTK_BIN (main_gui.popup_play_menu)->child),play_menu_name);
	g_free(play_menu_name);
			
	/* update statusbar */
	gtk_statusbar_pop (main_gui.statusbar1, 1);
	gtk_statusbar_push (main_gui.statusbar1, 1, gui_prefs.DefaultGame);
	gtk_statusbar_pop (main_gui.statusbar2, 1);
	if (rom->has_roms == 2)
		gtk_statusbar_push (main_gui.statusbar2, 1, _("UNKNOWN"));
	else if (rom->has_roms == 0)
		gtk_statusbar_push (main_gui.statusbar2, 1, _("ROMs missing"));
	else if (!rom->status)
		gtk_statusbar_push (main_gui.statusbar2, 1, _("Not working"));
	else
		gtk_statusbar_push (main_gui.statusbar2, 1, _("Working"));

	/* update screenshot panel */
	gtk_container_remove(GTK_CONTAINER(main_gui.screenshot_hist_vbox),main_gui.testgtkpix);
#ifdef DEBUG	
	gtk_container_remove(GTK_CONTAINER(main_gui.screenshot_hist_vbox),main_gui.history_box);
#endif
			
	filename = g_strdup_printf("%s%s.png",gui_prefs.SnapshotDirectory,rom->romname);
	pixbuf = gdk_pixbuf_new_from_file (filename);
	/* no picture found try parent game if any*/
	if ((pixbuf == NULL) && strcmp(rom->cloneof,"-"))
	{
		filename = g_strdup_printf("%s%s.png",gui_prefs.SnapshotDirectory,rom->cloneof);
		pixbuf = gdk_pixbuf_new_from_file (filename);
	}
	if (pixbuf != NULL)
	{
		gdk_pixbuf_render_pixmap_and_mask(pixbuf,
					&testpix,
					&testmask,
					127);
					
		gdk_pixbuf_unref (pixbuf);
		
	}
	else
	{
		gxmame_debug("no picture, fall back to the default one");
		testpix = gdk_pixmap_create_from_xpm_d(MainWindow->window,
					&testmask,
					&transparent,
					mame_xpm);
	}
	main_gui.testgtkpix = gtk_pixmap_new (testpix,testmask);
	g_free(filename);

	gtk_box_pack_start(main_gui.screenshot_hist_vbox,main_gui.testgtkpix,TRUE,TRUE,5);
	gtk_widget_show (main_gui.testgtkpix);
#ifdef DEBUG	
	main_gui.history_box = gtk_text_new(NULL,NULL);
	gtk_box_pack_start(main_gui.screenshot_hist_vbox,main_gui.history_box,TRUE,TRUE,5);
	gtk_widget_show (main_gui.history_box);
#endif
	
}


void show_progress_bar(void)
{
	if(gui_prefs.ShowStatusBar)
	{
		gtk_widget_hide (GTK_WIDGET(main_gui.tri_status_bar));
		gtk_statusbar_push(main_gui.status_progress_bar, 1, _("Game search 0%% complete"));
		gtk_widget_show (GTK_WIDGET(main_gui.combo_progress_bar));
	}	
}


void hide_progress_bar(void)
{
	if(gui_prefs.ShowStatusBar)
	{
		gtk_widget_hide (GTK_WIDGET(main_gui.combo_progress_bar));
		gtk_statusbar_pop(main_gui.status_progress_bar, 1);
		gtk_widget_show (GTK_WIDGET(main_gui.tri_status_bar));
	}	
}

void update_progress_bar(gfloat current_value)
{
	static gint current_displayed_value;
	gchar *displayed_message;
	if (current_displayed_value!=(gint)(current_value*100))
	{
		current_displayed_value=(gint)(current_value*100);
		displayed_message = g_strdup_printf(_("Game search %i%% complete"),current_displayed_value);
		gtk_statusbar_pop(main_gui.status_progress_bar, 1);
		gtk_statusbar_push(main_gui.status_progress_bar, 1, displayed_message);
	}
	gtk_progress_bar_update(main_gui.progress_progress_bar,current_value);
}

void update_game_in_list (guint position)
{
	gchar **Texte;
	gchar *Textes;
	GList *current_column;
	GdkPixmap *pixmap_unknown;
	GdkBitmap *mask_unknown;
	GdkPixmap *pixmap_rom;
	GdkBitmap *mask_rom;
	GdkPixmap *pixmap_norom;
	GdkBitmap *mask_norom;
	GdkPixmap *pixmap_redx;
	GdkBitmap *mask_redx;
	GdkColor  transparent;
	RomEntry *tmprom;
	
	tmprom = (RomEntry *)gtk_clist_get_row_data(GTK_CLIST(main_gui.displayed_list),position);		
	/* is the *[] really needed? a simple * could be enough? */
	Texte = g_malloc(sizeof(gchar *));
	
	if (tmprom->the_trailer && gui_prefs.ModifyThe)
		Texte[0] = g_strstrip(g_strdup_printf("%s, The %s",tmprom->gamename,tmprom->gamenameext));
	else if (tmprom->the_trailer && !gui_prefs.ModifyThe)
		Texte[0] = g_strstrip(g_strdup_printf("The %s %s",tmprom->gamename,tmprom->gamenameext));
	else
		Texte[0] = g_strstrip(g_strdup_printf("%s %s",tmprom->gamename,tmprom->gamenameext));

 	pixmap_unknown = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_unknown,
 				&transparent,
 				unknownS_xpm);
 	pixmap_rom = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_rom,
 				&transparent,
 				romS_xpm);
 	pixmap_norom = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_norom,
 				&transparent,
 				noromsS_xpm);
 	pixmap_redx = gdk_pixmap_create_from_xpm_d(
 				MainWindow->window,
 				&mask_redx,
 				&transparent,
 				redxS_xpm);
	
	gxmame_debug("update game display small icons mode");
	if (tmprom->has_roms == 2)
		gtk_clist_set_pixtext(GTK_CLIST(main_gui.displayed_list),
	                      position,
	                      0,
	                      Texte[0],
	                      5,
	                      pixmap_unknown,
	                      mask_unknown);
	else if (tmprom->has_roms == 0)
		gtk_clist_set_pixtext(GTK_CLIST(main_gui.displayed_list),
	                      position,
	                      0,
	                      Texte[0],
	                      5,
	                      pixmap_norom,
	                      mask_norom);
	else if (!tmprom->status)
		gtk_clist_set_pixtext(GTK_CLIST(main_gui.displayed_list),
	                      position,
	                      0,
	                      Texte[0],
	                      5,
	                      pixmap_redx,
	                      mask_redx);
	else
		gtk_clist_set_pixtext(GTK_CLIST(main_gui.displayed_list),
	                      position,
	                      0,
	                      Texte[0],
	                      5,
	                      pixmap_rom,
	                      mask_rom);
	if (gui_prefs.current_mode == DETAILS)
	{
		int i;
		gxmame_debug("update game display details mode");
		for(current_column = g_list_first(Columns),i=0;
				(current_column != NULL);
				current_column = g_list_next(current_column),i++)
		{
			switch (GPOINTER_TO_INT(current_column->data))
			{
				case (HAS_ROMS):
//					Textes = g_strdup_printf(tmprom->has_roms?_("Yes"):_("No"));
					Textes = g_strndup(tmprom->has_roms?_("Yes"):_("No"), MAX_GAMENAME);
					gtk_clist_set_text(GTK_CLIST(main_gui.displayed_list),
								position,i,Textes);
					break;
				case (HAS_SAMPLES):
					if (tmprom->nb_samples==0)
//						Textes = g_strdup_printf(" ");
						Textes = g_strndup(" ", MAX_GAMENAME);
					else 
					{
//						Textes = g_strdup_printf(tmprom->has_samples?_("Yes"):_("No"));
						Textes = g_strndup(tmprom->has_samples?_("Yes"):_("No"), MAX_GAMENAME);
					}
					gtk_clist_set_text(GTK_CLIST(main_gui.displayed_list),
								position,i,Textes);
					break;
				case (TIMESPLAYED):
					Textes = g_strdup_printf("%d", tmprom->timesplayed);
					gtk_clist_set_text(GTK_CLIST(main_gui.displayed_list),
								position,i,Textes);
					break;
			}
			
		}
		
	}
	select_game(tmprom);
	/* if we are in the favorite folder, and favorite flag of the rom is not set
	  we need to remove it from the CList (that imply that the game is in the CList )*/
	/* problems because the row values are completly changed, I redisplay the complete game list
	if ((current_filter->type == FAVORITE) && !tmprom->favourite)
	{
		gxmame_debug("removing %s from favorite list from position %i",tmprom->romname,position);
		gtk_clist_remove(GTK_CLIST(main_gui.displayed_list),position);
	}
	*/
}

